@@ -312,7 +312,7 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			 */
 			close(session->client_socket);
 			close_bindings(session->config->binding);
-			log_close(session->config->first_host, true);
+			log_close_for_cgi_run(session->config->first_host);
 
 			set_environment(session, NULL);
 
@@ -394,9 +394,10 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 }
 
 t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
-	bool read_again;
-	fd_set read_fds;
 	int bytes_read, result = cgi_OKE;
+	struct timeval select_timeout;
+	fd_set read_fds;
+	bool read_again;
 
 	do {
 		read_again = false;
@@ -405,7 +406,9 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 		FD_SET(cgi_info->from_cgi, &read_fds);
 		FD_SET(cgi_info->cgi_error, &read_fds);
 
-		switch (select(cgi_info->highest_fd, &read_fds, NULL, NULL, &(cgi_info->select_timeout))) {
+		select_timeout.tv_sec = 1;
+		select_timeout.tv_usec = 0;
+		switch (select(cgi_info->highest_fd, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				if (errno != EINTR) {
 					return cgi_ERROR;
@@ -415,11 +418,9 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			case 0:
 				if (session->force_quit) {
 					return cgi_FORCE_QUIT;
-				} else if (--(cgi_info->timer) <= 0) {
+				} else if (time(NULL) > cgi_info->deadline) {
 					return cgi_TIMEOUT;
 				} else {
-					cgi_info->select_timeout.tv_sec = 1;
-					cgi_info->select_timeout.tv_usec = 0;
 					read_again = true;
 				}
 		}
@@ -548,6 +549,7 @@ int send_fcgi_request(t_session *session, int sock) {
 
 static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buffer, int size) {
 	int bytes_read, result = cgi_ERROR;
+	struct timeval select_timeout;
 	fd_set read_fds;
 	bool read_again;
 
@@ -557,17 +559,17 @@ static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, c
 		FD_ZERO(&read_fds);
 		FD_SET(cgi_info->from_cgi, &read_fds);
 
-		switch (select(cgi_info->from_cgi + 1, &read_fds, NULL, NULL, &(cgi_info->select_timeout))) {
+		select_timeout.tv_sec = 1;
+		select_timeout.tv_usec = 0;
+		switch (select(cgi_info->from_cgi + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				break;
 			case 0:
 				if (session->force_quit) {
 					result = cgi_FORCE_QUIT;
-				} else if (--(cgi_info->timer) <= 0) {
+				} else if (time(NULL) > cgi_info->deadline) {
 					result = cgi_TIMEOUT;
 				} else {
-					cgi_info->select_timeout.tv_sec = 1;
-					cgi_info->select_timeout.tv_usec = 0;
 					read_again = true;
 				}
 				break;
