@@ -27,8 +27,7 @@ typedef enum { cgi_TIMEOUT = -3, cgi_FORCE_QUIT, cgi_ERROR, cgi_OKE, cgi_END_OF_
 
 typedef struct type_cgi_info {
 	t_cgi_type type;
-	int timer;
-	struct timeval select_timeout;
+	time_t deadline;
 
 	int to_cgi, from_cgi, cgi_error, highest_fd;
 
