@@ -506,7 +506,7 @@ int register_wrong_password(t_session *session) {
 				if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 					ban_ip(&(session->ip_address), session->config->ban_on_wrong_password, session->config->kick_on_ban);
 					session->keep_alive = false;
-					log_system(session, "Client banned because too many wrong passwords");
+					log_system(session, "Client banned because of too many wrong passwords");
 				}
 			}
 
