@@ -231,7 +231,7 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 
 #ifdef HAVE_SSL
 	if (session->binding->use_ssl) {
-		if (get_client_certificate(session->ssl_data, SSL_VAR_SIZE, subject, issuer) == 0) {
+		if (get_client_certificate(session->ssl_data, subject, issuer, SSL_VAR_SIZE) == 0) {
 			add_to_environment(fcgi_buffer, "SSL_CLIENT_SUBJECT", subject);
 			add_to_environment(fcgi_buffer, "SSL_CLIENT_ISSUER", issuer);
 		}
