@@ -79,6 +79,8 @@ typedef struct {
 } t_settings;
 
 volatile int received_signal = rs_NONE;
+bool               must_quit = false;
+
 char *hs_conlen      = "Content-Length: ";
 char *fb_filesystem  = "access denied via filesystem";
 char *fb_symlink     = "symlink not allowed";
@@ -314,18 +316,17 @@ int run_alter_program(t_session *session) {
 int fetch_request(t_session *session) {
 	char *new_reqbuf, *strstart, *strend;
 	long bytes_read, header_length = -1, content_length = -1;
-	int result = 200, timer, write_bytes;
+	int result = 200, write_bytes;
+	time_t deadline;
 	fd_set read_fds;
 	struct timeval select_timeout;
 	bool keep_reading = true, store_on_disk = false;
 	int upload_handle = -1;
 
-	select_timeout.tv_sec = 1;
-	select_timeout.tv_usec = 0;
 	if (session->kept_alive == 0) {
-		timer = session->binding->time_for_1st_request;
+		deadline = time(NULL) + session->binding->time_for_1st_request;
 	} else {
-		timer = session->binding->time_for_request;
+		deadline = time(NULL) + session->binding->time_for_request;
 	}
 
 #ifdef DEBUG
@@ -442,6 +443,9 @@ int fetch_request(t_session *session) {
 #endif
 		FD_ZERO(&read_fds);
 		FD_SET(session->client_socket, &read_fds);
+
+		select_timeout.tv_sec = 1;
+		select_timeout.tv_usec = 0;
 		switch (select(session->client_socket + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				if (errno != EINTR) {
@@ -453,12 +457,9 @@ int fetch_request(t_session *session) {
 				if (session->force_quit) {
 					result = fe_FORCE_QUIT;
 					keep_reading = false;
-				} else if (--timer <= 0) {
+				} else if (time(NULL) > deadline) {
 					result = fe_TIMEOUT;
 					keep_reading = false;
-				} else {
-					select_timeout.tv_sec = 1;
-					select_timeout.tv_usec = 0;
 				}
 				break;
 			default:
@@ -1232,6 +1233,71 @@ void connection_handler(t_session *session) {
 	pthread_exit(NULL);
 }
 
+/* Task-runner starts periodic tasks
+ */
+void task_runner(t_config *config) {
+	t_ip_addr ip_addr;
+
+	do {
+		sleep(1);
+
+		/* Client checks
+		 */
+		check_ban_list(config);
+		check_delayed_remove_timers(config);
+		remove_wrong_password_list(config);
+
+		/* FastCGI check
+		 */
+		check_load_balancer(config);
+
+		/* AccessLogfile check
+		 */
+		log_close(config->first_host, false);
+
+#ifdef HAVE_CACHE
+		/* Cache check
+		 */
+		check_cache();
+#endif
+
+#ifdef HAVE_COMMAND
+		/* CommandChannel check
+		 */
+		check_admin_list();
+#endif
+
+		switch (received_signal) {
+			case rs_NONE:
+				break;
+			case rs_QUIT_SERVER:
+				must_quit = true;
+				break;
+			case rs_UNBAN_CLIENTS:
+				default_ipv4(&ip_addr);
+				unban_ip(&ip_addr);
+#ifdef HAVE_IPV6
+				default_ipv6(&ip_addr);
+				unban_ip(&ip_addr);
+#endif
+				received_signal = rs_NONE;
+				break;
+			case rs_UNLOCK_LOGFILES:
+				log_close(config->first_host, true);
+				received_signal = rs_NONE;
+				break;
+#ifdef HAVE_CACHE
+			case rs_CLEAR_CACHE:
+				clear_cache();
+				received_signal = rs_NONE;
+				break;
+#endif
+		}
+	} while (must_quit == false);
+
+	pthread_exit(NULL);
+}
+
 /* Signal handlers
  */
 void SEGV_handler(int sig) {
@@ -1449,18 +1515,22 @@ int accept_connection(t_binding *binding, t_config *config) {
 
 		if (pthread_attr_init(&child_attr) != 0) {
 			log_system(session, "pthread init error");
-		} else if (pthread_attr_setdetachstate(&child_attr, PTHREAD_CREATE_DETACHED) != 0) {
-			log_system(session, "pthread set detach state error");
-		} else if (pthread_attr_setstacksize(&child_attr, PTHREAD_STACK_SIZE) != 0) {
-			log_system(session, "pthread set stack size error");
-		} else if (add_client(session) == 0) {
-			if (pthread_create(&child_thread, &child_attr, (void*)connection_handler, (void*)session) == 0) {
-				/* Thread started
-				 */
-				kick_client = false;
-			} else {
-				remove_client(session, false);
+		} else {
+			if (pthread_attr_setdetachstate(&child_attr, PTHREAD_CREATE_DETACHED) != 0) {
+				log_system(session, "pthread set detach state error");
+			} else if (pthread_attr_setstacksize(&child_attr, PTHREAD_STACK_SIZE) != 0) {
+				log_system(session, "pthread set stack size error");
+			} else if (add_client(session) == 0) {
+				if (pthread_create(&child_thread, &child_attr, (void*)connection_handler, (void*)session) == 0) {
+					/* Thread started
+					 */
+					kick_client = false;
+				} else {
+					remove_client(session, false);
+					log_system(session, "pthread create error");
+				}
 			}
+			pthread_attr_destroy(&child_attr);
 		}
 	} else switch (total_conns) {
 		case ca_TOOMUCH_PERIP:
@@ -1494,7 +1564,9 @@ int accept_connection(t_binding *binding, t_config *config) {
  */
 int run_server(t_settings *settings) {
 	int                highest_fd;
-	bool               must_quit = false;
+	struct timeval     select_timeout;
+	pthread_attr_t     task_runner_attr;
+	pthread_t          task_runner_thread;
 #ifdef HAVE_COMMAND
 	t_admin            *admin;
 	struct sockaddr_in caddr;
@@ -1508,10 +1580,8 @@ int run_server(t_settings *settings) {
 	t_binding          *binding;
 	t_config           *config;
     fd_set             read_fds;
-	struct timeval     select_timeout;
 	struct stat        status;
 	mode_t             access_rights;
-	t_ip_addr          ip_addr;
 
 	config = default_config();
 	if (chdir(settings->config_dir) == -1) {
@@ -1561,7 +1631,7 @@ int run_server(t_settings *settings) {
 	binding = config->binding;
 	while (binding != NULL) {
 		if (binding->use_ssl) {
-			if ((binding->ssl_context = ssl_binding(binding->server_key, binding->ca_cert, binding->verify_depth, config->dh_file, config->allowed_ciphers)) == NULL) {
+			if ((binding->ssl_context = ssl_binding(binding->ssl_key_cert, binding->required_ca, binding->verify_depth, config->dh_file, config->allowed_ciphers)) == NULL) {
 				perror("bind https");
 				return EXIT_FAILURE;
 			}
@@ -1707,9 +1777,6 @@ int run_server(t_settings *settings) {
 	}
 #endif
 
-	select_timeout.tv_sec = MAIN_LOOP_TIMER;
-	select_timeout.tv_usec = 0;
-
 	init_log_module();
 	init_client_module();
 	init_load_balancer(config->fcgi_server);
@@ -1750,12 +1817,34 @@ int run_server(t_settings *settings) {
 
 	log_string(config->system_logfile, "Hiawatha v"VERSION" started");
 
+	/* Start task_runner
+	 */
+	if (pthread_attr_init(&task_runner_attr) != 0) {
+		fprintf(stderr, "Task-runner pthread init error");
+		return EXIT_FAILURE;
+	} else if (pthread_attr_setdetachstate(&task_runner_attr, PTHREAD_CREATE_DETACHED) != 0) {
+		fprintf(stderr, "Task-runner pthread set detach state error");
+		return EXIT_FAILURE;
+	} else if (pthread_attr_setstacksize(&task_runner_attr, PTHREAD_STACK_SIZE) != 0) {
+		fprintf(stderr, "Task-runner pthread set stack size error");
+		return EXIT_FAILURE;
+	} else if (pthread_create(&task_runner_thread, &task_runner_attr, (void*)task_runner, (void*)config) != 0) {
+		fprintf(stderr, "Task-runner pthread create error");
+		return EXIT_FAILURE;
+	}
+	pthread_attr_destroy(&task_runner_attr);
+
+	/* Main loop
+	 */
 	do {
 #ifdef HAVE_COMMAND
 		highest_fd = fill_read_fds(&read_fds, config->binding, config->command_port);
 #else
 		highest_fd = fill_read_fds(&read_fds, config->binding);
 #endif
+
+		select_timeout.tv_sec = 1;
+		select_timeout.tv_usec = 0;
 		switch (select(highest_fd + 1, &read_fds, NULL, NULL, &select_timeout)) {
 			case -1:
 				if (errno != EINTR) {
@@ -1764,61 +1853,6 @@ int run_server(t_settings *settings) {
 				}
 				break;
 			case 0:
-				select_timeout.tv_sec = MAIN_LOOP_TIMER;
-				select_timeout.tv_usec = 0;
-
-				/* Client checks
-				 */
-				check_ban_list(config);
-				check_delayed_remove_timers(config);
-				remove_wrong_password_list(config);
-
-				/* FastCGI check
-				 */
-				check_load_balancer(config);
-
-				/* AccessLogfile check
-				 */
-				log_close(config->first_host, false);
-
-#ifdef HAVE_CACHE
-				/* Cache check
-				 */
-				check_cache();
-#endif
-
-#ifdef HAVE_COMMAND
-				/* CommandChannel check
-				 */
-				check_admin_list();
-#endif
-
-				switch (received_signal) {
-					case rs_NONE:
-						break;
-					case rs_QUIT_SERVER:
-						must_quit = true;
-						break;
-					case rs_UNBAN_CLIENTS:
-						default_ipv4(&ip_addr);
-						unban_ip(&ip_addr);
-#ifdef HAVE_IPV6
-						default_ipv6(&ip_addr);
-						unban_ip(&ip_addr);
-#endif
-						received_signal = rs_NONE;
-						break;
-					case rs_UNLOCK_LOGFILES:
-						log_close(config->first_host, true);
-						received_signal = rs_NONE;
-						break;
-#ifdef HAVE_CACHE
-					case rs_CLEAR_CACHE:
-						clear_cache();
-						received_signal = rs_NONE;
-						break;
-#endif
-				}
 				break;
 			default:
 #ifdef HAVE_COMMAND
@@ -1861,10 +1895,8 @@ int run_server(t_settings *settings) {
 								usleep(1000);
 								break;
 							}
-						} else {
-							if (add_admin(admin_socket) == -1) {
-								close(admin_socket);
-							}
+						} else if (add_admin(admin_socket) == -1) {
+							close(admin_socket);
 						}
 					}
 					binding = binding->next;
