@@ -4,14 +4,15 @@
 #include <stdbool.h>
 #include <openssl/bio.h>
 #include <openssl/ssl.h>
+#include <openssl/engine.h>
 
 int  ssl_init(char *buffer, int size);
-SSL_CTX *ssl_binding(char *keyfile, char *CA_cert, int verify_depth, char *dh_file, char *ciphers);
+SSL_CTX *ssl_binding(char *keyfile, char *ca_cert, int verify_depth, char *dh_file, char *ciphers);
 int  ssl_accept(int sock, SSL **ssl, SSL_CTX *context);
 int  ssl_receive(SSL *ssl, char *buffer, unsigned int maxlength);
 int  ssl_send(SSL *ssl, char *buffer, unsigned int length);
 int  ssl_close(SSL **ssl);
 void ssl_free(SSL_CTX *context);
-int  get_client_certificate(SSL *ssl_data, int size, char *subject, char *issuer);
+int  get_client_certificate(SSL *ssl_data, char *subject, char *issuer, int size);
 
 #endif
