@@ -169,8 +169,8 @@ static t_binding *new_binding(void) {
 		default_ipv4(&(binding->interface));
 #ifdef HAVE_SSL
 		binding->use_ssl              = false;
-		binding->server_key           = NULL;
-		binding->ca_cert              = NULL;
+		binding->ssl_key_cert         = NULL;
+		binding->required_ca          = NULL;
 		binding->verify_depth         = 1;
 		binding->ssl_context          = NULL;
 #endif
@@ -228,9 +228,6 @@ t_config *default_config(void) {
 		config->server_string      = "Hiawatha v"VERSION;
 		init_groups(&(config->groups));
 #ifdef HAVE_SSL
-		config->server_key         = CONFIG_DIR"/serverkey.pem";
-		config->ca_cert            = NULL;
-		config->verify_depth       = 1;
 		config->dh_file            = NULL;
 		config->allowed_ciphers    = NULL;
 #endif
@@ -273,6 +270,7 @@ t_config *default_config(void) {
 		config->banlist_mask       = NULL;
 		config->fcgi_server        = NULL;
 		config->upload_directory   = UPLOAD_DIR;
+		config->upload_directory_len = strlen(UPLOAD_DIR);
 		config->create_upload_dir  = false;
 #ifdef HAVE_TOOLKIT
 		config->url_toolkit        = NULL;
@@ -520,24 +518,15 @@ int check_configuration(t_config *config) {
 	t_host *host;
 	char c;
 	int i;
-#ifdef HAVE_SSL
 	t_binding *binding;
 
-	config->upload_directory_len = strlen(config->upload_directory);
-
 	binding = config->binding;
 	while (binding != NULL) {
-		if (binding->use_ssl && (binding->server_key == NULL)) {
-			binding->server_key = config->server_key;
-			binding->ca_cert = config->ca_cert;
-			binding->verify_depth = config->verify_depth;
-		}
 		if (binding->enable_alter) {
 			config->create_upload_dir = true;
 		}
 		binding = binding->next;
 	}
-#endif
 
 	if (config->first_host->hostname.size == 0) {
 		fprintf(stderr, "The default website has no hostname.\n");
@@ -833,20 +822,6 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if ((config->reconnect_delay = str2int(value)) != -1) {
 			return true;
 		}
-#ifdef HAVE_SSL
-/*
-	} else if (strcmp(key, "requiredca") == 0) {
-		split_string(value, &value, &rest, ',');
-		if ((config->ca_cert = strdup(value)) != NULL) {
-			if (rest != NULL) {
-				if ((config->verify_depth = str2int(rest)) < 1) {
-					return false;
-				}
-			}
-			return true;
-		}
-*/
-#endif
 	} else if (strcmp(key, "serverid") == 0) {
 		split_string(value, &uid, &gid, ':');
 		if (parse_userid(uid, &(config->server_uid)) == 1) {
@@ -860,12 +835,6 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				}
 			}
 		}
-#ifdef HAVE_SSL
-	} else if (strcmp(key, "serverkey") == 0) {
-		if ((config->server_key = strdup(value)) != NULL) {
-			return true;
-		}
-#endif
 	} else if (strcmp(key, "serverroot") == 0) {
 		if (valid_directory(value)) {
 			if ((config->server_root = strdup(value)) != NULL) {
@@ -916,6 +885,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	} else if (strcmp(key, "uploaddirectory") == 0) {
 		if (valid_directory(value)) {
 			if ((config->upload_directory = strdup(value)) != NULL) {
+				config->upload_directory_len = strlen(config->upload_directory);
 				return true;
 			}
 		}
@@ -1128,11 +1098,6 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_yesno(value, &(host->execute_cgi)) == 0) {
 			return true;
 		}
-	} else if (strcmp(key, "fastcgi") == 0) {
-		if (parse_charlist(value, &(host->fast_cgi)) != -1) {
-			host->execute_cgi = true;
-			return true;
-		}
 	} else if (strcmp(key, "followsymlinks") == 0) {
 		if (parse_yesno(value, &(host->follow_symlinks)) == 0) {
 			return true;
@@ -1169,6 +1134,11 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_charlist(value, &(host->required_binding)) == 0) {
 			return true;
 		}
+	} else if (strcmp(key, "usefastcgi") == 0) {
+		if (parse_charlist(value, &(host->fast_cgi)) != -1) {
+			host->execute_cgi = true;
+			return true;
+		}
 #ifdef HAVE_TOOLKIT
 	} else if (strcmp(key, "usetoolkit") == 0) {
 		if (parse_charlist(value, &(host->toolkit_rules)) == 0) {
@@ -1374,10 +1344,9 @@ static bool binding_setting(char *key, char *value, t_binding *binding) {
 			}
 		}
 #ifdef HAVE_SSL
-/*
 	} else if (strcmp(key, "requiredca") == 0) {
 		split_string(value, &value, &rest, ',');
-		if ((binding->ca_cert = strdup(value)) != NULL) {
+		if ((binding->required_ca = strdup(value)) != NULL) {
 			if (rest != NULL) {
 				if ((binding->verify_depth = str2int(rest)) < 1) {
 					return false;
@@ -1385,9 +1354,8 @@ static bool binding_setting(char *key, char *value, t_binding *binding) {
 			}
 			return true;
 		}
-*/
-	} else if (strcmp(key, "serverkey") == 0) {
-		if ((binding->server_key = strdup(value)) != NULL) {
+	} else if (strcmp(key, "sslcertfile") == 0) {
+		if ((binding->ssl_key_cert = strdup(value)) != NULL) {
 			binding->use_ssl = true;
 			return true;
 		}
@@ -1403,12 +1371,6 @@ static bool binding_setting(char *key, char *value, t_binding *binding) {
 			binding->time_for_1st_request = binding->time_for_request;
 			return true;
 		}
-#ifdef HAVE_SSL
-	} else if (strcmp(key, "usessl") == 0) {
-		if (parse_yesno(value, &(binding->use_ssl)) == 0) {
-			return true;
-		}
-#endif
 	}
 
 	return false;
@@ -1599,33 +1561,32 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 				 */
 				switch (section) {
 					case binding:
-						if (current_binding->port != -1) {
-#ifdef HAVE_SSL
-							if ((current_binding->server_key == NULL) && (current_binding->ca_cert != NULL)) {
-								fprintf(stderr, "RequiredCA requires ServerKey to be set in %s.\n", configfile);
-								retval = -1;
-							} else
-#endif
-								current_binding = NULL;
-						} else {
+						if (current_binding->port == -1) {
 							fprintf(stderr, "A Port is missing in a binding section in %s.\n", configfile);
 							retval = -1;
+#ifdef HAVE_SSL
+						} else if ((current_binding->ssl_key_cert == NULL) && (current_binding->required_ca != NULL)) {
+							fprintf(stderr, "RequiredCA requires ServerKey to be set in %s.\n", configfile);
+							retval = -1;
+#endif
+						} else {
+							current_binding = NULL;
 						}
 						break;
 					case directory:
-						if (config->directory->path != NULL) {
-							current_directory = NULL;
-						} else {
+						if (config->directory->path == NULL) {
 							fprintf(stderr, "A Path is missing in a directory section in %s.\n", configfile);
 							retval = -1;
+						} else {
+							current_directory = NULL;
 						}
 						break;
 					case fcgi_server:
-						if ((config->fcgi_server->fcgi_id != NULL) && (config->fcgi_server->connect_to != NULL)) {
-							current_fcgi_server = NULL;
-						} else {
+						if ((config->fcgi_server->fcgi_id == NULL) || (config->fcgi_server->connect_to == NULL)) {
 							fprintf(stderr, "A FastCGIid or ConnectTo is missing in a FastCGIserver section in %s.\n", configfile);
 							retval = -1;
+						} else {
+							current_fcgi_server = NULL;
 						}
 						break;
 					case virtual_host:
@@ -1641,11 +1602,11 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 						break;
 #ifdef HAVE_TOOLKIT
 					case url_toolkit:
-						if (current_toolkit->toolkit_id != NULL) {
-							current_toolkit = NULL;
-						} else {
+						if (current_toolkit->toolkit_id == NULL) {
 							fprintf(stderr, "A ToolkitID is missing in a UrlToolkit section in %s\n", configfile);
 							retval = -1;
+						} else {
+							current_toolkit = NULL;
 						}
 						break;
 #endif
