@@ -72,9 +72,9 @@ typedef struct type_binding {
 	char          *binding_id;
 #ifdef HAVE_SSL
 	bool          use_ssl;
-	char          *server_key;
+	char          *ssl_key_cert;
 	SSL_CTX       *ssl_context;
-	char          *ca_cert;
+	char          *required_ca;
 	int           verify_depth;
 #endif
 
@@ -203,9 +203,6 @@ typedef struct type_config {
 	t_log_format  log_format;
 	bool          wait_for_cgi;
 #ifdef HAVE_SSL
-	char          *server_key;
-	char          *ca_cert;
-	int           verify_depth;
 	char          *dh_file;
 	char          *allowed_ciphers;
 #endif
