@@ -12,7 +12,7 @@
 #define MAX_CHILD_ID        30000
 #define OUTPUT_BUFFER_SIZE      2 * KILOBYTE
 
-typedef enum { no_cgi, program, script, fastcgi} t_cgi_type;
+typedef enum { no_cgi, binary, script, fastcgi} t_cgi_type;
 typedef enum { unknown, GET, POST, HEAD, OPTIONS, TRACE, PUT, DELETE, unsupported } t_req_method;
 typedef enum { missing_slash, require_ssl, location } t_cause_of_301;
 
